<?php

/**
 * @file
 * Navigate UI Framework
 */
 
/**
 * Output a toggle button
 */
function navigate_ui_button($button) {
  if (!is_array($button)) {
    $button = array($button);
  }
  foreach ($button as $key => $value) {
    // Make value an array with at least a title
    if (!is_array($value)) {
      $value = array('title' => $value);
    }
    if (isset($value['name'])) {
      $value['id'] = 'navigate-ui-button-' . $value['name'];
    }
    else {
      $value['id'] = 'navigate-ui-button-' . $key;
    }
    return theme('navigate_ui_button', $value);
  }
}

function navigate_button($button_settings) {
  $defaults = array(
    'on' => '1',
    'off' => '0',
    'help' => '',
  );
  foreach ($defaults as $key => $val) {
    if (!key_exists($key, $button_settings)) {
      $button_settings[$key] = $val;
    }
  }
  $settings = navigate_widget_settings_get($button_settings['wid']);

  // If the setting hasn't set before, set it to the default
  if (!isset($settings[$button_settings['name']]) && $button_settings['default'] != '') {
    $settings = array(
      'name' => $button_settings['name'],
      'value' => $button_settings['default'],
      'wid' => $button_settings['wid'],
    );
    navigate_variable_set($settings);
    $settings = navigate_widget_settings_get($button_settings['wid'], TRUE);
  }

  $on_class = '';
  if ($settings[$button_settings['name']] == $button_settings['on']) {
    $on_class = ' navigate-button-on ';
  }
  $group_class = '';
  $group_input = '';
  if (isset($button_settings['group'])) {
    $group_class = ' '. $button_settings['group'] .' ';
    $group_input = '<input type="hidden" class="group" value="'. $button_settings['group'] .'" />';
  }
  $required = '';
  if (isset($button_settings['required'])) {
    $required = 1;
  }
  $callback = '';
  if (isset($button_settings['callback'])) {
    $callback = $button_settings['callback'];
  }
  $output = '
    <div class="navigate-button-outer">
      <div id="'. $button_settings['class'] .'-'. $button_settings['wid'] .'" class="navigate-button '. $button_settings['class'] . $on_class . $group_class .' navigate-tooltip">'. $button_settings['content'] .'</div>
      <div class="navigate-absolute">
        <input type="hidden" class="widget_type" value="'. $button_settings['widget_type'] .'" />
        <input type="hidden" class="on" value="'. $button_settings['on'] .'" />
        <input type="hidden" class="off" value="'. $button_settings['off'] .'" />
        <input type="hidden" class="name" value="'. $button_settings['name'] .'" />
        <input type="hidden" class="required" value="'. $required .'" />
        <input type="hidden" class="callback" value="'. $callback .'" />
        '. navigate_tooltip($button_settings['class'] .'-'. $button_settings['wid'], $button_settings['help']) .'
        '. $group_input .'
      </div>
    </div>';
  return $output;
}

 
/**
 * Associate a js callback with clicking content
 */
function navigate_callback_button($settings) {
  $settings['wid'] = isset($settings['wid']) ? $settings['wid'] : '';
  $output = '
    <span id="navigate-button-' . $settings['id'] . '-' . $settings['wid'] . '" class="navigate-button" title="' . $settings['tooltip'] . '">'. $settings['value'] .
    '<input type="hidden" class="navigate-data" value="' . htmlspecialchars(navigate_to_js($settings)) . '"></input></span>';
  return $output;
}

/**
 * Output a text input
 */
function navigate_input($input_settings) {
  $settings = navigate_widget_settings_get($input_settings['wid']);
  if ($input_settings['select_all']) {
    $select_all_class = ' navigate-select-all ';
  }
  $clear_class = '';
  if (isset($input_settings['clear'])) {
    $clear_class = ' navigate-clear ';
  }
  $settings[$input_settings['name']] = isset($settings[$input_settings['name']]) ? ($settings[$input_settings['name']]) : '';
  $output = '
    <div class="navigate-text-input-outer">
      <div class="navigate-text-input-container"><input id="'. $input_settings['class'] .'_'. $input_settings['wid'] .'" class="navigate-text-input '. $input_settings['class'] . $select_all_class . $clear_class .' navigate-tooltip" value="'. $settings[$input_settings['name']] .'" name="'. $input_settings['class'] .'" /></div>
      <input type="hidden" class="navigate-data" value="' . htmlspecialchars(drupal_to_js($input_settings)) . '"></input>
      '. navigate_tooltip($input_settings['class'] .'_'. $input_settings['wid'], $input_settings['help']) .'
    </div>';
  return $output;
}

/**
 * Output a textarea input
 */
function navigate_textarea($input_settings) {
  $filters_output = '';
  $input_settings['select_all'] = isset($input_settings['select_all']) ? $input_settings['select_all'] : '';
  $settings = navigate_widget_settings_get($input_settings['wid']);

  $select_all_class = isset($input_settings['select_all']) ? ' navigate-select-all ' : '';

  $filters_output = '';
  if (!isset($input_settings['hide_filters'])) {
    $filters = filter_formats();
    foreach ($filters as $filter) {
      if (!isset($default)) {
        $default = $filter->format;
      }
      $filters_output .= navigate_button(array(
        'name' => $input_settings['name'] .'_format',
        'content' => $filter->name,
        'class' => 'navigate-filter-'. $filter->format,
        'group' => $input_settings['name'] .'_format',
        'default' => $default,
        'help' => '',
        'on' => $filter->format,
        'required' => TRUE,
        'wid' => $input_settings['wid'],
      ));
    }
    if ($filters_output != '') {
      $filters_output = '<div class="navigate-filters">'. $filters_output .'</div>';
    }
  }

  if (!isset($input_settings['submit'])) {
    $input_settings['submit'] = 'Save';
  }

  $settings[$input_settings['name']] = isset($settings[$input_settings['name']]) ? $settings[$input_settings['name']] : '';

  $output = '
    <div class="navigate-textarea-outer">
      <div class="navigate-textarea-container">
        <textarea id="'. $input_settings['class'] .'_'. $input_settings['wid'] .'" class="navigate-textarea '. $input_settings['class'] . $select_all_class .' navigate-tooltip" name="'. $input_settings['class'] .'" rows="4" cols="30">'. htmlentities($settings[$input_settings['name']]) .'</textarea>
        '. navigate_tooltip($input_settings['class'] .'_'. $input_settings['wid'], $input_settings['help']) .'
        '. $filters_output .'
        <div class="navigate-submit '. $input_settings['class'] .'_submit_'. $input_settings['wid'] .'">'. t($input_settings['submit']) .'</div>
      </div>
      <input type="hidden" class="navigate-data" value="' . htmlspecialchars(drupal_to_js($input_settings)) . '"></input><
    </div>';
  return $output;
}

function navigate_tooltip($id, $content) {
  return '<div id="'. $id .'_tip_content" style="display:none">'. $content .'</div>';
}


function navigate_ui_data($data) {
  if (!is_array($data)) {
    $data = array($data);
  }
  if (!empty($data)) {
    return '<input type="hidden" class="navigate-data" value=\'' . navigate_to_js($data) . '\'></input>';
  }
  else {
    return '';
  }
}