<?php

/**
 * @file
 * Navigate Widget Framework
 */
 
/**
 * Add a new widget
 */
function navigate_widget_add($plugin = NULL) {
  global $_navigate, $user;
  if ($plugin && isset($_navigate['plugin']['default']['widgets'][$plugin])) {
    $widget = $_navigate['plugin']['default']['widgets'][$plugin];
    $wid = &$_navigate['users'][$user->uid]['settings']['widgetIndex'];
    $wid++;
    $widget['wid'] = $wid;
    $_navigate['users'][$user->uid]['widgets'][$wid] = $widget;    
    navigate_user_save($user->uid);
    return $widget;
  }
  else {
    return FALSE;
  }
}

/**
 * Build all widgets for a particular user
 */
function navigate_widget_build_all($uid = NULL) {
  global $_navigate, $user;
  if (!$uid) $uid = $user->uid;
  $output = '';
  if ($uid) {
    $widgets = $_navigate['users'][$uid]['widgets'];
    if (!empty($widgets)) {
      uasort($widgets, 'navigate_sort_weights');
      foreach ($widgets as $widget) {
        if (module_exists($widget['plugin']['name'])) {
          $output .= navigate_widget_render($widget);
        }
      }
    }
  }
  return $output;
}
 
/**
 * Deletes a widget
 */
function navigate_widget_delete($data) {
  global $_navigate, $user;
  $access = navigate_user_access(array('navigate' => 'widget remove'));
  if ($access) {
    $uid = $user->uid;
    if (isset($data['uid'])) {
      $uid = $data['uid'];
    }
    navigate_user_load($uid, TRUE);
    $wid = $data['wid'];
    $widgets = &$_navigate['users'][$uid]['widgets'];
    if (isset($widgets[$wid])) {
      unset($widgets[$wid]);
    }
    navigate_ajax_message(navigate_user_save($uid),
      t('Widget successfully deleted.'),
      t('Was unable to successfully delete widget.')
    );
  }
  else {
    navigate_ajax_message(FALSE, NULL, t('You do not have permissions to delete this widget.'));
  }
}

/**
 * Render a widget
 */
function navigate_widget_render($widget) {
  $output = '';
  $function = $widget['plugin']['name'] .'_navigate_plugin';
  if (function_exists($function)) {
    $output = $function('display', $widget);
    $widget['content'] = isset($output['content']) ? $output['content'] : '';
    $widget['settings'] = isset($output['settings']) ? $output['settings'] : array();
    $obj = new StdClass;
    $obj->content     = $widget['content'];
    $obj->customize   = (user_access('navigate customize') ? ' customize' : '');
    $obj->data        = navigate_ui_data(array('wid' => $widget['wid'], 'plugin' => $widget['plugin']['name']));
    $obj->plugin      = $widget['plugin'];
    $obj->processing  = base_path() . drupal_get_path('module', 'navigate') . '/images/processing-widget.gif';
    $obj->settings    = theme(array('navigate_widget_settings_' . $widget['wid'], 'navigate_widget_settings'), $widget['settings']);
    $widget_buttons = navigate_ui_button(array(
      'widget-' . $widget['wid'] . '-settings' => array(
        'title' => 'Widget Settings',
        'type' => 'checkbox',
        'data'  => array(
          'wid' => $widget['wid'],
          'callback' => 'toggleWidgetSettings',
          'jqueryui' => array(
            'icons' => array(
              'primary' => 'ui-icon-gear',
            ),
            'text' => FALSE,
          ),
        ),
      ),
    ));
    $obj->header      = '<span class="navigate-widget-title">' . $widget['title'] . '</span><span class="navigate-widget-buttons">' . $widget_buttons . '</span>';
    $obj->wid         = $widget['wid'];
    $output = theme(array('navigate_widget_' . $widget['wid'], 'navigate_widget'), $obj);
  }
  return $output;
}


/**
 * Save the new sort order for the widgets
 */
function navigate_widget_sort($data) {
  global $_navigate, $user;
  $access = navigate_user_access(array('navigate' => 'widget sort'));
  if ($access) {
    // Load user
    $uid = $user->uid;
    if (isset($data['uid'])) {
      $uid = $data['uid'];
    }
    navigate_user_load($uid, TRUE);
    // Set widget weights
    $widgets = &$_navigate['users'][$uid]['widgets'];
    foreach ($data['sort'] as $weight => $wid) {
      if (isset($widgets[$wid])) {
        $widgets[$wid]['weight'] = $weight;
      }
    }
    // Save user data with new widget weights
    navigate_ajax_message(navigate_user_save($uid),
      t('Widgets sorted successfully.'),
      t('Was unable to successfully sort widgets.')
    );
  }
  else {
    navigate_ajax_message(FALSE, NULL, t('You do not have permissions to sort widgets.'));
  }
  navigate_json($json);
}
